<?php

declare(strict_types=1);

namespace Dedi\SyliusPayboxPlugin\Payum\Action;

use Dedi\SyliusPayboxPlugin\Payum\PayboxApi;
use Payum\Core\Action\ActionInterface;
use Payum\Core\ApiAwareInterface;
use Payum\Core\ApiAwareTrait;
use Payum\Core\Bridge\Spl\ArrayObject;
use Payum\Core\Exception\RequestNotSupportedException;
use Payum\Core\Request\Capture;
use Psr\Log\LoggerAwareInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

final class CaptureAction implements ActionInterface, ApiAwareInterface, LoggerAwareInterface
{
    use ApiAwareTrait;

    protected LoggerInterface $logger;

    public function __construct(private readonly RequestStack $requestStack)
    {
        $this->apiClass = PayboxApi::class;
    }

    public function execute($request)
    {
        RequestNotSupportedException::assertSupports($this, $request);

        $status = $this->requestStack->getCurrentRequest()->get('error_code', null);

        /** @var ArrayObject $details */
        $details = $request->getModel();

        $query = $this->requestStack->getCurrentRequest()->getQueryString();
        $this->logger->info(sprintf('[%s] %s', PayboxApi::CODE, self::class), ['response' => $query, 'payment' => (array) $details]);

        if (null === $status) {
            $this->api->doPayment((array) $details);
        } else {
            $details->replace(['status' => $status]);
        }
    }

    public function setLogger(LoggerInterface $logger)
    {
        $this->logger = $logger;
    }

    public function supports($request)
    {
        return
            $request instanceof Capture &&
            $request->getModel() instanceof ArrayObject
        ;
    }
}
